/*
 * +-----------------------------------------------+
 * | System:                                       |
 * | Package:                                      |
 * | Class:    Comm                                |
 * +-----------------------------------------------+
 * | Author:   Michael Ransburg (Daneel)           |
 * |           <daneel@tux.org>                    |
 * | Created:  04.12.1999                          |
 * | Version:  0.0.1                               |
 * +-----------------------------------------------+
 *
 * Changes:
 *     04.12.1999 - (Daneel): - created :-)
 *
 */

import java.util.*;
import java.io.*;
import java.net.*;

public class Comm {

  public short length;
  public short messid = -1;
  public String content = new String();

  public int timeout = 1000;

  public Vector vecContent = new Vector();

  public InputStream from_server_raw;
  public DataOutputStream to_server;
  public DataInputStream from_server_be;
  public LEDataInputStream from_server_le;


  private int loggedin = 0;
  private String user;
  private String link;  
  private int localport;

  
  public int loggedIn() {
	return loggedin;
  }

  public int getPort() {
	return localport;
  }

  public String getUserName() {
	return user;  
  }

  public String getLink() {
	return link;
  }


  public void sendPacket(int mes, String text) {
  
	try {

	  byte[] data = new byte[2 + 2 + text.length()];

	  int msg_len = (short)text.length();
	  int msg_type = (short)mes;

	  data[0]     = (byte) msg_len;
	  data[1]     = (byte) (msg_len >> 8);
	  data[2]     = (byte) msg_type;
	  data[3]     = (byte) (msg_type >> 8);

	  System.arraycopy(text.getBytes(), 0, data, 4, text.length());	

	  to_server.write(data);
	  to_server.flush();
	}
  
	catch(IOException io) {
	  System.out.println(io);	
	}
  }



  public void recPacket() {

	try {
	
	  System.out.println("reached");

	  int tempint = (int) from_server_raw.read();
	  int tempint2 = (int) from_server_raw.read();
	  length = (short) ((tempint2 << 8) | tempint);

	  tempint = (int) from_server_raw.read();
	  tempint2 = (int) from_server_raw.read();
	  messid = (short) ((tempint2 << 8) | tempint);

	  byte[] textBuf = new byte[length];
  
  	  int read = 0;
	  while (read < length) {
		read += from_server_raw.read(textBuf, read, length - read);
	  }
	  
	  System.out.println("reached2");

	  content = new String(textBuf);
	  
	  System.out.println("\nReading from 0 to " + length + " (ID: " + messid);	  
	  System.out.println("Content: " + content + "\n\n");

	}
	catch(IOException io) {
	  System.out.println(io);
	}

  }



  public void recPacket(int id, int delay) {
  // next packet with id "id". sets messid to -10 if no more packages.

	int available;
	int factor = 0;
	int nomorepackages = 0;
	byte[] textBuf;
	int tempint;
	int tempint2;
	int read;
  
	try {

	  System.out.println("reached");

      byte[] buffer;  

	  available = from_server_raw.available();

	  for ( ;; ) {
	  
		available = from_server_raw.available();

		if (available != 0) { //there is a package waiting

		  tempint = (int) from_server_raw.read();
		  tempint2 = (int) from_server_raw.read();
		  length = (short) ((tempint2 << 8) | tempint);

		  tempint = (int) from_server_raw.read();
		  tempint2 = (int) from_server_raw.read();
		  messid = (short) ((tempint2 << 8) | tempint);

		
		  while (messid != id) { //it does not have the correct id - discard packages
		  
			textBuf = new byte[length];

  			read = 0;
			while (read < length) {
			  read += from_server_raw.read(textBuf, read, length - read);
			}


			if (from_server_raw.available() == 0) { //no more packages waiting - break out of while loop
			  nomorepackages = 1;
			  break;
			}
		  
			tempint = (int) from_server_raw.read();
			tempint2 = (int) from_server_raw.read();
			length = (short) ((tempint2 << 8) | tempint);

			tempint = (int) from_server_raw.read();
			tempint2 = (int) from_server_raw.read();
			messid = (short) ((tempint2 << 8) | tempint);

			
		  }

		  // we should have a package with the correct id now
		  if (nomorepackages == 0) { //only read if a package is waiting

			textBuf = new byte[length];
  
  			read = 0;
			while (read < length) {
			  read += from_server_raw.read(textBuf, read, length - read);
			}

			content = new String(textBuf);
	  
		  }

		  // we have finished reading a correct package - let's break out of the loop
		  break;

		}
	  
		else { //there is no package waiting - let's wait a specific time
	  
		  java.lang.Thread.sleep(500);
		  factor++;
		  
		  if (factor == delay) { //we got a timeout --> no more packages with this id
	  
			messid = -10;
			break;		  

		  }
		}
	  }
	}
	catch(IOException io) {
	  System.out.println(io);
	}
	
	catch(InterruptedException ie) {
	  System.out.println(ie);
	}
  
  }


  public short getMessid() {

	return messid;
  
  }
  
  public String getContent() {

	return content;
  
  }

  public String LoginDistribution(String remotehost, int remoteport) { // log into distribution server and return fileserver:port

	char[] buffer = new char[1024];
	byte[] buffer2 = new byte[10000];
	int chars_read;

    // open the socket
    try {

	  int howmuch = 0;

  	  Socket s = new Socket(remotehost, remoteport);

	  // status
	  GUI.status.changeStatus("Logging in ...");

	  // define a reader / writer to read and write the socket
	  Reader from_server = new InputStreamReader(s.getInputStream());
	  PrintWriter to_server2 = new PrintWriter(new OutputStreamWriter(s.getOutputStream()));

	  // define a reader / writer to read and write the console
	  BufferedReader from_user = new BufferedReader(new InputStreamReader(System.in));
	  PrintWriter to_user = new PrintWriter(new OutputStreamWriter(System.out));

	  while((chars_read = from_server.read(buffer)) != -1) {
	    to_user.write(buffer, 0, chars_read);
	    to_user.flush();
	  }
	  to_user.print("\nGot IP from Server, connecting... ");
	  to_user.print("\n");
	  to_user.flush();

	  // close socket
	  s.close();
	  
	}

	catch(IOException e) {
	  GUI.status.changeStatus("Login failed.");
  	  System.out.println(e);
	}

	return new String(buffer);

  }


  public void LoginFileServer(String fileHost, int newuser, String username, String password, String port, String clientinfo, String linktype) {

	try {

	  // set some variables
	  user = username;
	  link = linktype;
	  localport = new Integer(port.trim()).intValue();


	  // we got ip:port from server. let's split that up
	  StringTokenizer tokenizer = new StringTokenizer(fileHost,":");
	  String remotehost = tokenizer.nextToken();
	  String temp = tokenizer.nextToken();

	  // best way to convert string --> integer
	  Integer remoteport = new Integer(temp.trim());


	  System.out.println("Connecting to " + remotehost + " " + remoteport.intValue());


	  // open a connection to the new ip
	  Socket t = new Socket(remotehost, remoteport.intValue());

	  // get inputstream for available() call in this.recpacket()
	  from_server_raw = t.getInputStream();

	  // define a reader / writer to read and write the socket
	  from_server_be = new DataInputStream(t.getInputStream());
	  from_server_le = new LEDataInputStream(t.getInputStream());

	  // for sending byte values and stuff
	  to_server = new DataOutputStream(t.getOutputStream());

	  if (newuser == 1) { //new user
		this.sendPacket(7, username);
		this.recPacket();
		if (messid != 8) {
		  GUI.status.changeStatus("Registration failed - username already exists.");
		  return;
		}

		String loginstring = username + " " + password + " " + port + " " + "\"" + clientinfo + "\"" + " " + linktype + " " + "email@dummy.com";
		this.sendPacket(6, loginstring);

	  }

	  else { //usual login

		String loginstring = username + " " + password + " " + port + " " + clientinfo + " " + linktype;
		this.sendPacket(2, loginstring);

	  }

	  this.recPacket();
	  
	  if (this.messid == 3) {
		GUI.status.changeStatus("Logged in successfully");
		loggedin = 1;
	  }
	  else {
		GUI.status.changeStatus("Failed to login");
		return;
	  }
		

	  this.recPacket(621, 11);
		
	  GUI.content.changecontent(" "); // clear text area

	  while (messid != -10) {

		GUI.content.appendcontent("                 ");
		GUI.content.appendcontent(content);		  		
		GUI.content.appendcontent("\n");
		System.out.println(content);		  

		this.recPacket(621, 11);
		System.out.println(messid);

  	  }
	}
	catch(IOException e) {
	  GUI.status.changeStatus("Login failed.");
  	  System.out.println(e);
	}
  }
}



